/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/

#include <SPI.h>
#include <Wire.h>
#include <stdio.h>
#include "ssd1357.h"

void command(uint8_t cmd) {
    digitalWrite(OLED_DC, LOW);
    SPIWrite(&cmd, 1);
}

void data(uint8_t dat) {
    digitalWrite(OLED_DC, HIGH);
    SPIWrite(&dat, 1);
}


void SPIWrite(uint8_t *buffer, int bufferLength) {
    int i;
    for (i = 0; i < bufferLength; i++) {
        SPI.transfer(buffer[i]);
    }
}

void SSD1357_begin()
{
    pinMode(OLED_RST, OUTPUT);
    pinMode(OLED_DC, OUTPUT);
    pinMode(OLED_CS, OUTPUT);
    SPI.begin();
      SPI.beginTransaction(SPISettings(40000000, MSBFIRST, SPI_MODE0));
    //SPI.setClockDivider(SPI_CLOCK_DIV2);

    digitalWrite(OLED_CS, LOW);
    
    digitalWrite(OLED_RST, HIGH);
    delay(10);
    digitalWrite(OLED_RST, LOW);
    delay(10);
    digitalWrite(OLED_RST, HIGH);

    command(0xfd);
    data(0x12);
    command(0xae);
    command(0xa0);
    data(0x71);
    data(0x00);
    command(0xa1);
    data(0x00);
    command(0xa2);
    data(0x00);
    command(0xb1);
    data(0x84);
    command(0xb3);//Set Display Clock Divide Ratio/Oscillator Frequency
    data(0x20);//105Hz
    command(0xb6);
    data(0x01);
    command(0xbb);
    data(0x00);
    command(0xbe);//Set VCOMH
    data(0x07);//0.86*VCC
    command(0xc7);//Master Contrast
    data(0x0f);//The value is for VCC=15V
    command(0xc1);//Contrast Current
    data(0x32);//B
    data(0x29);//G
    data(0x53);//R
    command(0xca);//Set MUX Ratio
    data(0x7f);//128 Duty
    command(0xaf); //Display on
 
}

void SSD1357_clear(uint16_t color)
{
    int i, j;
    for(i = 0; i < OLED_WIDTH; i++)
    {
        for(j = 0; j < OLED_WIDTH; j++)
        {
            SSD1357_draw_point(j, i, color);
        }
    }
}

void SSD1357_draw_point(int x, int y, unsigned short hwColor) {

    unsigned char buffer[2];
    
    if(x >= OLED_WIDTH || y >= OLED_HEIGHT)
    {
        return;
    }

    command(0x15);
    data(y+32);                
    data(y+32);    
    command(0x75);
    data(x);                 
    data(x);   
    command(0x5c);    
    
    digitalWrite(OLED_DC, HIGH);    
    buffer[0] = (hwColor & 0xFF00) >> 8;
    buffer[1] = hwColor & 0xFF;
    SPIWrite(buffer, 2);
}

void SSD1357_char1616(unsigned char x, unsigned char y, unsigned char chChar, unsigned short hwColor)
{
    unsigned char i, j;
    unsigned char chTemp = 0, y0 = y;

    for (i = 0; i < 32; i ++) {
        chTemp = pgm_read_byte(&Font1612[chChar - 0x30][i]);
        for (j = 0; j < 8; j ++) {
            if (chTemp & 0x80) {
                SSD1357_draw_point(x, y, hwColor);
            } else {
                SSD1357_draw_point(x, y, 0);
            }
            chTemp <<= 1;
            y++;
            if ((y - y0) == 16) {
                y = y0;
                x++;
                break;
            }
        }
    }
}

void SSD1357_char3216(unsigned char x, unsigned char y, unsigned char chChar, unsigned short hwColor)
{
    unsigned char i, j;
    unsigned char chTemp = 0, y0 = y; 

    for (i = 0; i < 64; i++) {
        chTemp = pgm_read_byte(&Font3216[chChar - 0x30][i]);
        for (j = 0; j < 8; j++) {
            if (chTemp & 0x80) {
                SSD1357_draw_point(x, y, hwColor);
            } else {
                SSD1357_draw_point(x, y, 0);
            }

            chTemp <<= 1;
            y++;
            if ((y - y0) == 32) {
                y = y0;
                x++;
                break;
            }
        }
    }
}

void SSD1357_char(unsigned char x, unsigned char y, char acsii, char size, char mode, unsigned short hwColor)
{
    unsigned char i, j, y0=y;
    char temp;
    unsigned char ch = acsii - ' ';
    for(i = 0;i<size;i++) {
        if(size == 12)
        {
            if(mode)temp = pgm_read_byte(&Font1206[ch][i]);
            else temp = ~pgm_read_byte(&Font1206[ch][i]);
        }
        else 
        {            
            if(mode)temp = pgm_read_byte(&Font1608[ch][i]);
            else temp = ~pgm_read_byte(&Font1608[ch][i]);
        }
        for(j =0;j<8;j++)
        {
            if(temp & 0x80) SSD1357_draw_point(x, y, hwColor);
            else SSD1357_draw_point(x, y, 0);
            temp <<=1;
            y++;
            if((y-y0)==size)
            {
                y = y0;
                x ++;
                break;
            }
        }
    }
}

void SSD1357_string(unsigned char x, unsigned char y, const char *pString, unsigned char Size, unsigned char Mode, unsigned short hwColor)
{
    while (*pString != '\0') {       
        if (x > (OLED_WIDTH - Size / 2)) {
            x = 0;
            y += Size;
            if (y > (OLED_HEIGHT - Size)) {
                y = x = 0;
            }
        }
        
        SSD1357_char(x, y, *pString, Size, Mode, hwColor);
        x += Size / 2;
        pString ++;
    }
}

void SSD1357_mono_bitmap(unsigned char x, unsigned char y, const unsigned char *pBmp, unsigned char chWidth, unsigned char chHeight, unsigned short hwColor)
{
    unsigned char i, j, byteWidth = (chWidth + 7) / 8;
    for(j = 0; j < chHeight; j++) {
        for(i = 0; i <chWidth; i ++) {
            if(pgm_read_byte(pBmp + j * byteWidth + i / 8) & (128 >> (i & 7))) {
                SSD1357_draw_point(x + i, y + j, hwColor);
            }
        }
    }        
}

void SSD1357_65k_bitmap(unsigned char x, unsigned char y, const unsigned char *pBmp, unsigned char chWidth, unsigned char chHeight)
{unsigned short hwColor;
    unsigned short i, j,tem;
    for(j = 0; j < chHeight; j++) {
        for(i = 0; i <chWidth; i ++) {
              hwColor=(pgm_read_byte(pBmp ++));
               tem=(pgm_read_byte(pBmp ++));
               hwColor<<=8;           
               hwColor=hwColor|tem;
               SSD1357_draw_point(x + i, y + j, hwColor);                    
            
        }
    }        
}


